# Gemini

_Requires Python 3 and Clingo 5. Both should be in your PATH._

_Also requires the following Python packages: numpy, deap, typing (`pip install numpy`, etc)._

## Quick Start

To generate games using Gemini, run this command on a terminal: 

`python simulate.py` _outfilename_ _m_ `generation/gemini.lp` _intent_ _n_ --project

Where 
- _outfilename_ is the output file prefix
- _n_ is the number of distinct (uninstantiated) games to generate using Clingo
- _m_ is the number of random games within the _n_ set to instantiate and optimize using the genetic algorithm in `simulate.py`
- _intent_ is the name of the ASP intent file to use for generation (should include any suffix, e.g., intent.lp)

Any Clingo command line argument can also be invoked anywhere after the first two arguments. 

## Examples

	python simulate.py temp 5 generation/gemini.lp intents/intent.lp 10 --project
	
Generates five distinct Cygnus games (`temp_1.lp`, ...,`temp_5.lp`), and copies each one 5 times, producing 25 total files* (such that `temp_1.lp` = `temp_6.lp` = `temp_11.lp` = `temp_16.lp` = `temp_21.lp`.). Those 5 distinct games were randomly chosen from a pool of 10 uninstantiated games generated by Clingo from the design intent, `intent.lp`. Those 10 games should be relatively distinct because we used the `--project` option, explained below. The 5 games selected from the 10 are then instantiated and optimized by `simulate.py`'s genetic algorithm. 

	python simulate.py temp 1 generation/gemini.lp intent.lp --project 0 -t 6

Generates one distinct Cygnus game and copies it to 25 files*, `temp_1.lp`, `temp_2.lp`, ..., `temp_25.lp`. This one game is randomly chosen (by `simulate.py`) from a pool of all possible games (as indicated by `0`) generated by Clingo from the design intent, `intent.lp`. It is then optimized/instantiated by the genetic algorithm run in `simulate.py`. Clingo is run with 6 threads for potentially faster solving. 

	clingo generation/gemini.lp intent.lp  | tr [:space:] \\n | sort
    
Generates one abstract, uninstantiated game satisfying `intent.lp`, and prints to the console all the game's definition facts, along with other Clingo output, separated by newlines and sorted. 

	clingo generation/gemini.lp intent.lp
    
Generates one abstract, uninstantiated game satisfying `intent.lp`, and prints to the console all the game's definition facts in a large block of text (which is fairly unreadable). 
    

## Gemini Commands Explained

**Command to run the full Gemini suite:**

	python simulate.py prefix number-of-games clingo-arguments
or

    python simulate.py prefix number-of-games generation/gemini.lp intent n
    
Generates 25 Cygnus game files with the given filename prefix, which were solved by Clingo with respect to the given problem definition and intent files.

**`prefix`** — Name to append an underscore, number, and .lp file extension to when creating the Cygnus files. E.g., `dinner` generates `dinner_1.lp`, `dinner_2.lp`, ..., `dinner_25.lp`.

**`number-of-games`** — Number of distinct games to be randomly chosen from the pool generated by Clingo, initialized/optimized by simulate.py's genetic algorithm, and output as Cygnus files. Should always be less than or equal to the number passed to Clingo for the maximum number of answer sets to solve for (`n` in the above command).

**`clingo-arguments`** — The command line arguments to run Clingo with. Should include the intent file, the main Gemini generation file (`generation/gemini.lp`), and any Clingo options. Should not include `clingo` itself.

- To generate one distinct Gemini game for a given intent, use `generation/gemini.lp intent.lp` as Clingo arguments.
- To generate all possible games for an intent, use `generation/gemini.lp intent.lp 0` (though this computation may take a while).
- To generate a maximum of 5 random games for an intent, use `generation/gemini.lp intent.lp 5`

**\*Note:** simulate.py is currently hard-coded to create 25 .lp files, regardless of the given `number-of-games` argument. If you optimize less than 25 games, they will be duplicated, after optimization, to produce 25 files. This is so you don't have to modify the list of .lp files read in by Coordinator.js for the climate change game project.


## Clingo

**A Clingo call takes this form:**

	clingo [ filenames | number | options ]

The filenames, number, and options do not need to be in any particular order. 

**`filenames`** — Any number of AnsProlog (.lp) files. 

**`number`** — A maximum number of models to compute (0 meaning all possible models). 1 if unspecified. Computing all possible answer sets can take a while, so use the `0` argument intentionally. Often, we use `3`, `5`, `10`, or `25` while developing and iterating on design intents. 

**`options`** — Clingo options. Some useful ones described below. 

#### Useful Clingo Options

**`--project`** — Project answer sets to named atoms and only enumerate unique projected solutions. Named project atoms are specified using the `#project` directive in the AnsProlog problem file(s). In Gemini, they're mostly in `generation.lp`. This option prevents a calculated pool of answer sets from being too homogenous. 

**`-t n`** — Enable parallel solving with _n_ threads. Not to exceed double the number of cores on your machine. 

Other options are described in the Potassco Handbook and via `clingo --help`.  

## Writing Design Intents

The current minimal design intent is one that gives constant integer values to `min_resources`, `max_resources`, `min_outcomes`, `max_outcomes`, and `min_end_outcomes`. 

For example,

```
#const min_resources = 1.
#const max_resources = 2.
#const min_outcomes = 2.
#const max_outcomes = 10.
#const min_end_outcomes = 0.
```

See files in `intents` for examples of other statements you can make in an intent. 
